@@ -1,5 +1,5 @@
 #include "clar_libgit2.h"
-#include "fileops.h"
+#include "futils.h"
 #include "path.h"
 #include "posix.h"
 
@@ -150,10 +150,11 @@ static void cleanup_chmod_root(void *ref)
 	git_futils_rmdir_r("r", NULL, GIT_RMDIR_EMPTY_HIERARCHY);
 }
 
-#define check_mode(X,A) check_mode_at_line((X), (A), __FILE__, __LINE__)
+#define check_mode(X,A) check_mode_at_line((X), (A), __FILE__, __func__, __LINE__)
 
 static void check_mode_at_line(
-	mode_t expected, mode_t actual, const char *file, int line)
+	mode_t expected, mode_t actual,
+	const char *file, const char *func, int line)
 {
 	/* FAT filesystems don't support exec bit, nor group/world bits */
 	if (!cl_is_chmod_supported()) {
@@ -162,7 +163,7 @@ static void check_mode_at_line(
 	}
 
 	clar__assert_equal(
-		file, line, "expected_mode != actual_mode", 1,
+		file, func, line, "expected_mode != actual_mode", 1,
 		"%07o", (int)expected, (int)(actual & 0777));
 }
 
