@@ -3,7 +3,7 @@
 #include "git2/merge.h"
 #include "merge.h"
 #include "refs.h"
-#include "fileops.h"
+#include "futils.h"
 
 static git_repository *repo;
 static git_index *repo_index;
