@@ -6,7 +6,7 @@
 #include "../merge_helpers.h"
 #include "../conflict_data.h"
 #include "refs.h"
-#include "fileops.h"
+#include "futils.h"
 
 static git_repository *repo;
 static git_index *repo_index;
